/*
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */
package com.edelvez.gameserver.handler;

import java.util.logging.Logger;

import javolution.util.FastMap;
import com.edelvez.Config;
import com.edelvez.gameserver.handler.chathandlers.ChatAll;
import com.edelvez.gameserver.handler.chathandlers.ChatAlliance;
import com.edelvez.gameserver.handler.chathandlers.ChatClan;
import com.edelvez.gameserver.handler.chathandlers.ChatHeroVoice;
import com.edelvez.gameserver.handler.chathandlers.ChatParty;
import com.edelvez.gameserver.handler.chathandlers.ChatPartyRoomAll;
import com.edelvez.gameserver.handler.chathandlers.ChatPartyRoomCommander;
import com.edelvez.gameserver.handler.chathandlers.ChatPetition;
import com.edelvez.gameserver.handler.chathandlers.ChatShout;
import com.edelvez.gameserver.handler.chathandlers.ChatTell;
import com.edelvez.gameserver.handler.chathandlers.ChatTrade;

/**
 * This class handles all chat handlers
 *
 * @author durgus and Maxi56
 */
public class ChatHandler
{
	private static Logger _log = Logger.getLogger(ChatHandler.class.getName());
	private FastMap<Integer, IChatHandler> _datatable;

	public static ChatHandler getInstance()
	{
		return SingletonHolder._instance;
	}

	private ChatHandler()
	{
		_datatable = new FastMap<Integer, IChatHandler>();
		registerChatHandler(new ChatAll());
		registerChatHandler(new ChatAlliance());
		registerChatHandler(new ChatClan());
		registerChatHandler(new ChatHeroVoice());
		registerChatHandler(new ChatParty());
		registerChatHandler(new ChatPartyRoomAll());
		registerChatHandler(new ChatPartyRoomCommander());
		registerChatHandler(new ChatPetition());
		registerChatHandler(new ChatShout());
		registerChatHandler(new ChatTell());
		registerChatHandler(new ChatTrade());
		_log.config("ChatHandler: Loaded " + _datatable.size() + " handlers.");
	}

	public void registerChatHandler(IChatHandler handler)
	{
		int[] ids = handler.getChatTypeList();
		for (int id : ids) {
			if (Config.DEBUG) {
				_log.fine("Adding handler for chat type " + id);
			}
			_datatable.put(id, handler);
		}
	}

	public IChatHandler getChatHandler(int chatType)
	{
		return _datatable.get(chatType);
	}

	public int size()
	{
		return _datatable.size();
	}

	private final static class SingletonHolder
	{
		protected static final ChatHandler _instance = new ChatHandler();
	}
}