/*
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */
package com.edelvez.gameserver.handler;

import java.util.logging.Logger;

import javolution.util.FastMap;
import com.edelvez.gameserver.handler.itemhandlers.BeastSoulShot;
import com.edelvez.gameserver.handler.itemhandlers.BeastSpice;
import com.edelvez.gameserver.handler.itemhandlers.BeastSpiritShot;
import com.edelvez.gameserver.handler.itemhandlers.BlessedSpiritShot;
import com.edelvez.gameserver.handler.itemhandlers.Book;
import com.edelvez.gameserver.handler.itemhandlers.CharChangePotions;
import com.edelvez.gameserver.handler.itemhandlers.ChestKey;
import com.edelvez.gameserver.handler.itemhandlers.CrystalCarol;
import com.edelvez.gameserver.handler.itemhandlers.DonatorItems;
import com.edelvez.gameserver.handler.itemhandlers.DungeonKeys;
import com.edelvez.gameserver.handler.itemhandlers.EnchantScrolls;
import com.edelvez.gameserver.handler.itemhandlers.EnergyStone;
import com.edelvez.gameserver.handler.itemhandlers.ExtractableItems;
import com.edelvez.gameserver.handler.itemhandlers.Firework;
import com.edelvez.gameserver.handler.itemhandlers.FishShots;
import com.edelvez.gameserver.handler.itemhandlers.Harvester;
import com.edelvez.gameserver.handler.itemhandlers.JackpotSeed;
import com.edelvez.gameserver.handler.itemhandlers.Maps;
import com.edelvez.gameserver.handler.itemhandlers.MercTicket;
import com.edelvez.gameserver.handler.itemhandlers.MysteryPotion;
import com.edelvez.gameserver.handler.itemhandlers.Potions;
import com.edelvez.gameserver.handler.itemhandlers.PrimevalPotions;
import com.edelvez.gameserver.handler.itemhandlers.Recipes;
import com.edelvez.gameserver.handler.itemhandlers.Remedy;
import com.edelvez.gameserver.handler.itemhandlers.RollingDice;
import com.edelvez.gameserver.handler.itemhandlers.ScrollOfEscape;
import com.edelvez.gameserver.handler.itemhandlers.ScrollOfResurrection;
import com.edelvez.gameserver.handler.itemhandlers.Scrolls;
import com.edelvez.gameserver.handler.itemhandlers.ScrollsValakas;
import com.edelvez.gameserver.handler.itemhandlers.Seed;
import com.edelvez.gameserver.handler.itemhandlers.SevenSignsRecord;
import com.edelvez.gameserver.handler.itemhandlers.SoulCrystals;
import com.edelvez.gameserver.handler.itemhandlers.SoulShots;
import com.edelvez.gameserver.handler.itemhandlers.SpecialXMas;
import com.edelvez.gameserver.handler.itemhandlers.SpiritShot;
import com.edelvez.gameserver.handler.itemhandlers.SummonItems;

/**
 * @author Maxi
 */
public class ItemHandler
{
	private static Logger _log = Logger.getLogger(ItemHandler.class.getName());
	private FastMap<Integer, IItemHandler> _datatable;

	public static ItemHandler getInstance()
	{
		return SingletonHolder._instance;
	}

	private ItemHandler()
	{
		_datatable = new FastMap<Integer, IItemHandler>();
		registerItemHandler(new ScrollOfEscape());
		registerItemHandler(new ScrollOfResurrection());
		registerItemHandler(new SoulShots());
		registerItemHandler(new SpiritShot());
		registerItemHandler(new BlessedSpiritShot());
		registerItemHandler(new BeastSoulShot());
		registerItemHandler(new BeastSpiritShot());
		registerItemHandler(new ChestKey());
		registerItemHandler(new DungeonKeys());
		registerItemHandler(new Maps());
		registerItemHandler(new Potions());
		registerItemHandler(new Recipes());
		registerItemHandler(new RollingDice());
		registerItemHandler(new MysteryPotion());
		registerItemHandler(new EnchantScrolls());
		registerItemHandler(new EnergyStone());
		registerItemHandler(new Book());
		registerItemHandler(new Remedy());
		registerItemHandler(new Scrolls());
		registerItemHandler(new CrystalCarol());
		registerItemHandler(new DonatorItems());
		registerItemHandler(new SoulCrystals());
		registerItemHandler(new SevenSignsRecord());
		registerItemHandler(new CharChangePotions());
		registerItemHandler(new Firework());
		registerItemHandler(new Seed());
		registerItemHandler(new Harvester());
		registerItemHandler(new MercTicket());
		registerItemHandler(new FishShots());
		registerItemHandler(new JackpotSeed());
		registerItemHandler(new ExtractableItems());
		registerItemHandler(new SpecialXMas());
		registerItemHandler(new SummonItems());
		registerItemHandler(new BeastSpice());
		registerItemHandler(new PrimevalPotions());
		registerItemHandler(new ScrollsValakas());
		_log.config("ItemHandler: Loaded " + _datatable.size() + " handlers.");
	}

	public void registerItemHandler(IItemHandler handler)
	{
		// Get all ID corresponding to the item type of the handler
		int[] ids = handler.getItemIds();
		// Add handler for each ID found
		for (int id : ids) {
			_datatable.put(new Integer(id), handler);
		}
	}

	public IItemHandler getItemHandler(int itemId)
	{
		return _datatable.get(new Integer(itemId));
	}

	public int size()
	{
		return _datatable.size();
	}

	private final static class SingletonHolder
	{
		protected static final ItemHandler _instance = new ItemHandler();
	}
}
