/*
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */
package com.edelvez.gameserver.handler;

import java.util.logging.Logger;

import javolution.util.FastMap;
import com.edelvez.gameserver.handler.skillhandlers.BalanceLife;
import com.edelvez.gameserver.handler.skillhandlers.BeastFeed;
import com.edelvez.gameserver.handler.skillhandlers.Blow;
import com.edelvez.gameserver.handler.skillhandlers.Charge;
import com.edelvez.gameserver.handler.skillhandlers.CombatPointHeal;
import com.edelvez.gameserver.handler.skillhandlers.Continuous;
import com.edelvez.gameserver.handler.skillhandlers.CpDam;
import com.edelvez.gameserver.handler.skillhandlers.Craft;
import com.edelvez.gameserver.handler.skillhandlers.DeluxeKey;
import com.edelvez.gameserver.handler.skillhandlers.Disablers;
import com.edelvez.gameserver.handler.skillhandlers.DrainSoul;
import com.edelvez.gameserver.handler.skillhandlers.Fishing;
import com.edelvez.gameserver.handler.skillhandlers.FishingSkill;
import com.edelvez.gameserver.handler.skillhandlers.GetPlayer;
import com.edelvez.gameserver.handler.skillhandlers.GiveSp;
import com.edelvez.gameserver.handler.skillhandlers.Harvest;
import com.edelvez.gameserver.handler.skillhandlers.Heal;
import com.edelvez.gameserver.handler.skillhandlers.ManaHeal;
import com.edelvez.gameserver.handler.skillhandlers.Manadam;
import com.edelvez.gameserver.handler.skillhandlers.Mdam;
import com.edelvez.gameserver.handler.skillhandlers.Pdam;
import com.edelvez.gameserver.handler.skillhandlers.Recall;
import com.edelvez.gameserver.handler.skillhandlers.Resurrect;
import com.edelvez.gameserver.handler.skillhandlers.SiegeFlag;
import com.edelvez.gameserver.handler.skillhandlers.Sow;
import com.edelvez.gameserver.handler.skillhandlers.Spoil;
import com.edelvez.gameserver.handler.skillhandlers.StrSiegeAssault;
import com.edelvez.gameserver.handler.skillhandlers.SummonFriend;
import com.edelvez.gameserver.handler.skillhandlers.SummonTreasureKey;
import com.edelvez.gameserver.handler.skillhandlers.Sweep;
import com.edelvez.gameserver.handler.skillhandlers.TakeCastle;
import com.edelvez.gameserver.handler.skillhandlers.Unlock;
import com.edelvez.gameserver.handler.skillhandlers.ZakenPlayer;
import com.edelvez.gameserver.handler.skillhandlers.ZakenSelf;
import com.edelvez.gameserver.model.L2Skill;
import com.edelvez.gameserver.model.L2Skill.SkillType;

/**
 * @author Maxi56
 */
public class SkillHandler
{
	private static Logger _log = Logger.getLogger(SkillHandler.class.getName());
	private FastMap<L2Skill.SkillType, ISkillHandler> _datatable;

	public static SkillHandler getInstance()
	{
		return SingletonHolder._instance;
	}

	private SkillHandler()
	{
		_datatable = new FastMap<L2Skill.SkillType, ISkillHandler>();
		registerSkillHandler(new Blow());
		registerSkillHandler(new Pdam());
		registerSkillHandler(new Mdam());
		registerSkillHandler(new CpDam());
		registerSkillHandler(new Manadam());
		registerSkillHandler(new Heal());
		registerSkillHandler(new CombatPointHeal());
		registerSkillHandler(new ManaHeal());
		registerSkillHandler(new BalanceLife());
		registerSkillHandler(new Charge());
		registerSkillHandler(new Continuous());
		registerSkillHandler(new Resurrect());
		registerSkillHandler(new Spoil());
		registerSkillHandler(new Sweep());
		registerSkillHandler(new SummonTreasureKey());
		registerSkillHandler(new Disablers());
		registerSkillHandler(new Recall());
		registerSkillHandler(new Unlock());
		registerSkillHandler(new DrainSoul());
		registerSkillHandler(new Craft());
		registerSkillHandler(new Fishing());
		registerSkillHandler(new FishingSkill());
		registerSkillHandler(new BeastFeed());
		registerSkillHandler(new DeluxeKey());
		registerSkillHandler(new Sow());
		registerSkillHandler(new Harvest());
		registerSkillHandler(new GetPlayer());
		registerSkillHandler(new GiveSp());
		registerSkillHandler(new ZakenPlayer());
		registerSkillHandler(new ZakenSelf());
		registerSkillHandler(new StrSiegeAssault());
		registerSkillHandler(new SummonFriend());
		registerSkillHandler(new SiegeFlag());
		registerSkillHandler(new TakeCastle());
		_log.config("SkillHandler: Loaded " + _datatable.size() + " handlers.");
	}

	public void registerSkillHandler(ISkillHandler handler)
	{
		SkillType[] types = handler.getSkillIds();
		for (SkillType t : types)
		{
			_datatable.put(t, handler);
		}
	}

	public ISkillHandler getSkillHandler(SkillType skillType)
	{
		return _datatable.get(skillType);
	}

	public int size()
	{
		return _datatable.size();
	}

	private final static class SingletonHolder
	{
		protected static final SkillHandler _instance = new SkillHandler();
	}
}
